local _, ns = ...

local _G = _G
local setfenv = _G.setfenv
setfenv(1, ns)
local assert = _G.assert
local ipairs = _G.ipairs
local CreateFrame = _G.CreateFrame
local GetNumPartyMembers = _G.GetNumPartyMembers
local IsPartyLeader = _G.IsPartyLeader

local function debug(msg)
    _G.ChatFrame1:AddMessage(msg)
end

local initFunc, onEvent, onTabChange, onPartyChange

local enableSingleButton = function()
    local singleButton = _G.PVPFrameLeftButton
    local singleText = _G.PVPFrameLeftButtonText
    singleButton:Enable()
    singleText:SetTextColor(1.00,0.82,0.00,1.00)
end

local disableSingeButton = function()
    local singleButton = _G.PVPFrameLeftButton
    local singleText = _G.PVPFrameLeftButtonText
    singleButton:Disable()
    singleText:SetTextColor(0.5,0.5,0.5,0.5)
end

local setButtons = function()
    local isInParty = GetNumPartyMembers() > 0
    local isPartyLeader = IsPartyLeader()
    if isInParty then
        if isPartyLeader then
            disableSingeButton()
        else
            enableSingleButton()
        end
    else
        enableSingleButton()
    end
end

onTabChange = function(self)
    local singleButton = _G.PVPFrameLeftButton
    local singleText = _G.PVPFrameLeftButtonText
    local index = self:GetID()

    if index == 1 then
        setButtons()
    else
        enableSingleButton()
    end
end

initFunc = function()
    _G.hooksecurefunc('PVPFrame_TabClicked', onTabChange)
    initFunc = nil
    return true
end

onPartyChange = function()
    -- only relevant if frame is on honor tab
    local lastSelectedTab = _G.PVPFrame.lastSelectedTab
    if lastSelectedTab then
        local id = lastSelectedTab:GetID()
        if id == 1 then
            -- honor frame
            setButtons()
        else
            enableSingleButton()
        end
    else
        debug("no tab ever selected?")
    end
end

onEvent = function(self, event)
    --debug(event)
    if event == 'PLAYER_LOGIN' then
        local success = initFunc()
        assert(success)
        self:UnregisterEvent(event)
    --elseif event == 'PARTY_LEADER_CHANGED' then
        --debug(event)
    elseif (event == 'PARTY_MEMBERS_CHANGED' or event == 'RAID_ROSTER_UPDATE') then
        -- party / raid changes: check if we are still the leader
        --debug(event)
        onPartyChange()
    end
end

local eventFrame = CreateFrame('frame')
eventFrame:RegisterEvent('PLAYER_LOGIN')
--eventFrame:RegisterEvent('PARTY_LEADER_CHANGED')
eventFrame:RegisterEvent('PARTY_MEMBERS_CHANGED')
eventFrame:RegisterEvent('RAID_ROSTER_UPDATE')
eventFrame:SetScript('OnEvent', onEvent)

-- events fired                                    | PLC | PMC | RRU
-- -----------------------------------------------------------------
-- inviting someone if you are alone               | yes | yes | no
-- sometimes in between                            |     | yes |
-- invite from you to someone else times out       | no  |
-- someone else invites you                        | no  | no  | no
-- - you decline                                   | no  | yes | no
-- - you timeout                                   | no  | no  | no
-- - you accept                                    | yes | yes | no
-- you get party lead                              | yes | yes | no
-- you lose party lead                             | yes | yes | yes